/***************************************************************************
                          main.cpp  -  description
                             -------------------
    begin                : Don Jul 18 14:06:52 CEST 2002
    copyright            : (C) 2002 by Michael Majdic
    email                : michael.majdic@gmx.at
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <iostream.h>
#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <string.h>

#define MODE0 0
#define MODE1 1
#define MODE2 2
#define FALSE 0
#define TRUE  1
#define MAXFILELENGTH 256
#define MAXLINELENGTH 3004
#define MAXMAPPEDVECTORS 10000
#define MAXPHRASELENGTH 1024
#define MAXPHRASESPERUNIT 1000
#define MAXLABELS 200
#define MINGRAYVALUE 180

/* global */
// options
int mode =MODE1;
char model[256] = "kea_model";
int number_of_phrases =15;
char path[256] = "output/";
char keypath[12] ="output/key/";
char keylinkpath[64]="key/";
char keyfileending[5] =".key";
char sourcepath[64] ="output/files/";
char sourcelinkpath[256] = "files/";
int labelfiles_exist = 1;
char kea_tmp_path[MAXFILELENGTH] = "output/tmp_kea/";
char labelfiles_path[MAXFILELENGTH] = "output/labels/";
char htmlprefix[MAXFILELENGTH];
int keylinks_output = 0;
int labelsomI_output =0;

struct files
{
	char name[MAXFILELENGTH];
};
struct phrase
{
	int wc_phrase;
	char phrase[MAXPHRASELENGTH];
	char stemmed_phrase[MAXPHRASELENGTH];
	double score;
};


struct unit
{
	int pos_x;
	char pos_x_str[256];
	int pos_y;
	char pos_y_str[256];
	char node_id[256];
	char quanterror_node[64];
	int nr_mapped_vecs;
	files filelist[MAXMAPPEDVECTORS];
	int nr_soms_mapped;
	files url_mapped_soms;
	int nr_mode0_labels;
	int nr_mode1_labels;
	int nr_ghsom_labels;
	phrase m1_labels[MAXLABELS];
	phrase m0_labels[MAXLABELS];
	phrase ghsom_labels[MAXLABELS];
};

struct map
{
	int xdim;
	int ydim;
	int layer;
	char somabove[MAXFILELENGTH];	
};


// prototype
int generate_html(char labelfilename[MAXFILELENGTH]);
int write_html(unit curr_unit,map curr_map,FILE *htmlfile);
int calculate_color(double max_value, double min_value, double score);

int main(int argc, char *argv[])
{
	FILE *processfile;
	FILE *propfile; // option file
	char line[MAXFILELENGTH];
	char propfilename[MAXFILELENGTH];
	char labelfilename[MAXFILELENGTH];
	char *token;
	char help_string[256];
	char *substring;


	if(argc != 3)
	{
		fprintf(stderr,"Usage: %s processfile config_file\n", argv[0]);
		return 0;
	}


// read and parse the optionfile expected as the second command lien argument	

	propfile = fopen(argv[2],"r");
	while (fgets(line, MAXFILELENGTH, propfile) != NULL)
	{
  	token=strtok(line, " ");
 		while ((substring=strtok(NULL, " ")) != NULL)
		{
			if (strcmp(token, "MODE")==0)
    	{  // either mode0 mode1 or mode2 i.e. 0, 1 or 2 is expected
 				strcpy(help_string,substring);
    		mode = atol(help_string);	
				if ((mode != MODE0) && (mode != MODE1) && (mode != MODE2))
				{
					fprintf(stderr,"Invalid value for option MODE in given option file, ciao ...\n");
					return 0;
				}
    	}
			if (strcmp(token, "KEYLINKS_OUTPUT")==0)
    	{  // 0 = TRUE, everthing else considered as fasle, default is true
 				strcpy(help_string,substring);
    		keylinks_output = atol(help_string);	
    	}
			if (strcmp(token, "LABELSOMI_OUTPUT")==0)
    	{  // 0 = TRUE, everthing else considered as fasle, default is true
 				strcpy(help_string,substring);
    		labelsomI_output = atol(help_string);	
    	}
    	else if (strcmp(token, "KEA_MODEL")==0)
    	{  // the filename of the kea_model is expected
    		strcpy(model,substring);
				model[strlen(model)-1] = '\0';

    	}
   		else if (strcmp(token, "NUMBEROFPHRASES")==0)
    	{  // how many phrases should be extracted / used
    		strcpy(help_string,substring);
    		number_of_phrases = atol(help_string);	
    	}
    	else if (strcmp(token, "OUTPUTPATH")==0)
    	{  // the path where the unitfiles are expected and the modiefied file are written
    		strcpy(path,substring);
				path[strlen(path)-1] = '\0';
    	}
   		else if (strcmp(token, "KEYPATH")==0)
    	{  // the path where the 'keyfiles' are expected, i.e. the file generated by KEA
				strcpy(keypath,substring);
				keypath[strlen(keypath)-1] = '\0';
    	}
   		else if (strcmp(token, "KEAFILEEXTENSION")==0)
    	{  // the EXTENSION of the kea files, normally is should be .key
				strcpy(keyfileending,substring);
				keyfileending[strlen(keyfileending)-1] = '\0';
    	}
  		else if (strcmp(token, "SOURCEPATH")==0)
    	{  // the path, where the orginal sourcefiles can be found
				strcpy(sourcepath,substring);
				sourcepath[strlen(sourcepath)-1] = '\0';
    	}
  		else if (strcmp(token, "TMPKEAPATH")==0)
    	{  // a temporary path
				strcpy(kea_tmp_path,substring);
				kea_tmp_path[strlen(kea_tmp_path)-1] = '\0';
    	}
   		else if (strcmp(token, "KEYFILESLINKPATH")==0)
    	{  // the path where the 'keyfiles' are expected, i.e. the file generated by KEA
				strcpy(keylinkpath,substring);
				keylinkpath[strlen(keylinkpath)-1] = '\0';

    	}
   		else if (strcmp(token, "SOURCEFILESLINKPATH")==0)
    	{  // the path where the 'keyfiles' are expected, i.e. the file generated by KEA
				strcpy(sourcelinkpath,substring);
				sourcelinkpath[strlen(sourcelinkpath)-1] = '\0';
	   	}
    }
	}
  fclose(propfile);

//close optionfile and open the generated processfile

  processfile = fopen(argv[1],"r");
	while (fgets(line, MAXFILELENGTH, processfile) != NULL)
	{

  	memset(labelfilename,0,strlen(labelfilename) * sizeof(char));
  	strcpy(labelfilename, path);
  	strncat(labelfilename,line,strlen(line)-1);
   	if (generate_html(labelfilename)==0)
   	{
    		fprintf(stderr,"Error generating html code for %s\n", labelfilename);
     		return 0;
   	}
	}
	fclose(processfile);
	return 1;
}

int generate_html(char labelfilename[MAXFILELENGTH])
{
	FILE *labelfile;
	FILE *htmlfile;
	char labelpath[MAXFILELENGTH];
	char line[MAXLINELENGTH];
	char help_line[MAXLINELENGTH];
	char *token;
	char *help;
	char *substring;
	int 	is_begin_of_file=TRUE;
	unit current_unit;
	int old_pos_x;
	int old_pos_y;
	map current_map;
	int vec_counter = 0;
	int m0_counter = 0;
	int ghsom_counter =0;


  //strcpy(labelpath, path);
	//strcat(labelpath, labelfilename);
	fprintf(stdout,"Processing %s ....", labelfilename);
	if ((labelfile=fopen(labelfilename, "r"))==NULL)
	{
		fprintf(stderr,"Couldn't open labelfile: %s\n",labelfilename);
		return 0;
	}
	strcat(labelfilename,".html");
	fprintf(stdout,"and writing %s !\n", labelfilename);
	if ((htmlfile=fopen(labelfilename, "w"))==NULL)
	{
		fprintf(stderr,"Couldn't open htmlfile: %s\n",labelfilename);
		return 0;
	}
	//initalize variables
	old_pos_x = -1;
	old_pos_y = -1;
	current_map.layer = -1;
	current_unit.nr_mode0_labels = 0;
	current_unit.nr_mode1_labels = 0;
	current_unit.nr_ghsom_labels = 0;
	strcpy(current_map.somabove, "\0");

	while (fgets(line, MAXLINELENGTH, labelfile)!=NULL)
	{
		strcpy(help_line,line);
		token=strtok(line, "\n ");
		while ((substring=strtok(NULL, " ")) != NULL)
		{
	 		if (strcmp(token,"$NODE_ID")==0)
 			{ // write header just once
 				if (current_map.layer == -1)
 				{ // this important, because if i wouldn't check this
 				  // i would jump in the branch again with a different substring
 					help=strtok(substring,"_");
	 				help=strtok(NULL,"_");
					current_map.layer = atol(help);
				}
 			}
			else if (strcmp(token, "$SOMABOVE")==0)
     	{  // SOM ABOVE
    		strcpy(current_map.somabove,substring);
     	} 			
			else if (strcmp(token, "$XDIM")==0)
     	{  // X-Dimension of the map
    		current_map.xdim = atol(substring);
     	}
 			else if (strcmp(token, "$YDIM")==0)
     	{  // Y-Dimension of the map
	   		current_map.ydim = atol(substring);
     	}
			
 			else if (strcmp(token, "$POS_X")==0)
     	{  // new unit because X-Pos is given
  			if (is_begin_of_file==TRUE)
  			{
  				is_begin_of_file = FALSE;
  			}
  			else
  			{
  				if (m0_counter < current_unit.nr_mode0_labels)
  				{
  					current_unit.nr_mode0_labels=m0_counter;
  				}
  				if (write_html(current_unit, current_map, htmlfile)==FALSE)
  				{
  					fprintf(stderr,"Haternal Fatal Error in function write_html\n");
  					return 0;
  				}
  			}
  			//initalize current unit
 				current_unit.nr_mode0_labels = 0;
				current_unit.nr_mode1_labels = 0;
				current_unit.nr_ghsom_labels = 0;
				vec_counter=0;
				m0_counter=0;
				ghsom_counter=0;
				
  			strcpy(current_unit.pos_x_str,substring);
    		current_unit.pos_x = atol(current_unit.pos_x_str);
     	}
    	else if (strcmp(token, "$POS_Y")==0)
    	{  // Y-Position of the unit
    		strcpy(current_unit.pos_y_str,substring);
    		current_unit.pos_y = atol(current_unit.pos_y_str);
    	}
  		else if (strcmp(token, "$NR_VEC_MAPPED")==0)
  		{
  			current_unit.nr_mapped_vecs=atoi(substring);
  		}
  		else if (strcmp(token, "$VEC")==0)
  		{
  			strcpy(current_unit.filelist[vec_counter].name,substring);
  			// GET RID OF ANNOYING \n character
  			if (current_unit.filelist[vec_counter].name[strlen(current_unit.filelist[vec_counter].name)-1] == '\n')
  			{
  				current_unit.filelist[vec_counter].name[strlen(current_unit.filelist[vec_counter].name)-1] = '\0';
  			}
  			vec_counter++;
   		}
   		else if (strcmp(token, "$NR_SOMS_MAPPED")==0)
   		{
   			current_unit.nr_soms_mapped = atoi(substring);
   		}
   		else if (strcmp(token, "$URL_MAPPED_SOMS")==0)
   		{
   			strcpy(current_unit.url_mapped_soms.name, substring);
   		}
   		else if(strcmp(token, "$MODE0_PHRASES")==0)
   		{
   			current_unit.nr_mode0_labels=atoi(substring);
   		}
   		else if (strcmp(token, "$M0_LABEL")==0)
   		{
				if (m0_counter < MAXLABELS)
   			{
					token=strtok(help_line," ");
   				help=strtok(NULL,"\t");
   				strcpy(current_unit.m0_labels[m0_counter].phrase, help);
   				help=strtok(NULL,"\t");
   				strcpy(current_unit.m0_labels[m0_counter].stemmed_phrase, help);
   				help=strtok(NULL,"\t");
   				current_unit.m0_labels[m0_counter].score = atof(help);
   				m0_counter++;
				}
   		}
   		else if(strcmp(token, "$NR_LABELS")==0)
   		{
   			current_unit.nr_ghsom_labels=atoi(substring);
   		}
   		else if(strcmp(token, "$LABEL")==0)
   		{
   			strcpy(current_unit.ghsom_labels[ghsom_counter].phrase,substring);
   			ghsom_counter++;
   		}
   		else if (strcmp(token, "$MODE1_PHRASES")==0)
   		{
				current_unit.nr_mode1_labels =atoi(substring);
				if (current_unit.nr_mode1_labels > MAXLABELS)
				{
					current_unit.nr_mode1_labels = MAXLABELS;
				}
				for(int i=0; i<current_unit.nr_mode1_labels;i++)
				{
					fgets(line,MAXLINELENGTH,labelfile);
		//			fprintf(stdout, "%s\n", line);
					substring=strtok(line,"\t");
		//			fprintf(stdout, "%s\n", substring);
					strcpy(current_unit.m1_labels[i].phrase, substring);
					substring=strtok(NULL,"\t");
		//			fprintf(stdout, "%s\n", substring);
					strcpy(current_unit.m1_labels[i].stemmed_phrase, substring);
					substring=strtok(NULL,"\t");
		//			fprintf(stdout, "%s\n", substring);
					current_unit.m1_labels[i].score = atof(substring);
				}
   		}
  	}
  }
	if (write_html(current_unit, current_map, htmlfile)==FALSE)
  {
  	fprintf(stderr,"Haternal Fatal Error in function write_html\n");
  	return 0;
  }
	fclose(labelfile);
	fclose(htmlfile);	
	return 1;
}

int write_html(unit curr_unit,map curr_map,FILE *htmlfile)
{
	char mapped_som[MAXFILELENGTH];
	char help_mapped_som[MAXFILELENGTH];
	char somabove[MAXFILELENGTH];
	int color;
	
	if ((curr_unit.pos_x==0) && (curr_unit.pos_y==0))
	{ // write header
		fprintf(htmlfile,"<HTML><BODY bgcolor=#FFFFFF text=#000000><TITLE>Layer %d Map</TITLE>\n",curr_map.layer);
		fprintf(htmlfile,"<H1>Layer %d Map</H1><BR>\n",curr_map.layer);
		fprintf(htmlfile,"<BR><table border=8 cellpadding=4>\n");
	}
	if (curr_unit.pos_x == 0)
	{ //new line in the table
		fprintf(htmlfile,"<tr>\n");
	}
	fprintf(htmlfile,"<td align=left>\n");
	if ((curr_unit.nr_ghsom_labels != 0) && (labelsomI_output == 0))
	{// write ghsom labels
		fprintf(htmlfile,"LABELs by LabelSOM I<BR>");
		for (int i=0;i < curr_unit.nr_ghsom_labels;i++)
		{
			fprintf(htmlfile,"%s<BR>",curr_unit.ghsom_labels[i].phrase);
		}
		fprintf(htmlfile,"<HR>");
	}
	
	if (curr_unit.nr_mode1_labels != 0)
	{// write mode 1 labels
		fprintf(htmlfile,"LABELS by LabelSOM II<BR>");
		for (int i=0;i < curr_unit.nr_mode1_labels;i++)
		{
			  // calculate color value for label
				color = calculate_color(curr_unit.m1_labels[0].score,curr_unit.m1_labels[curr_unit.nr_mode1_labels-1].score,curr_unit.m1_labels[i].score);
				fprintf(htmlfile,"<b><font color=#%x%x%x>%s (%.4f)</font></b><BR>",color,color,color,curr_unit.m1_labels[i].phrase,curr_unit.m1_labels[i].score);
		}
		fprintf(htmlfile,"<HR>");
	}
	if (curr_unit.nr_mode0_labels != 0)
	{// write mode 0 labels
		fprintf(htmlfile,"Mode 0 LABELS<BR>");
		for (int i=0;i < curr_unit.nr_mode0_labels;i++)
		{
				// calculate color value (0 - 216) for the label
				color = calculate_color(curr_unit.m0_labels[0].score,curr_unit.m0_labels[curr_unit.nr_mode0_labels-1].score,curr_unit.m0_labels[i].score);
				fprintf(htmlfile,"<b><font color=#%x%x%x>%s (%.4f)</font></b><BR>",color,color,color,curr_unit.m0_labels[i].phrase,curr_unit.m0_labels[i].score);

		}
		fprintf(htmlfile,"<HR>");
	}
	
	if (curr_unit.nr_soms_mapped == 0)
	{ // its a leave and not a knot
		if (curr_unit.nr_mapped_vecs == 0)
		{ // there are obviosly no mapped vecs, labels or stuff, so write a point to the file
			fprintf(htmlfile,".");
		}
		else
		{ // make the links to the mapped files
		
			for (int i=0;i < curr_unit.nr_mapped_vecs;i++)
			{
				fprintf(htmlfile,"<a href=%s%s>%s</a><BR>",sourcelinkpath,curr_unit.filelist[i].name,curr_unit.filelist[i].name);
			}
			if (keylinks_output == 0)
			{	// if the option is set make the links to the keyfiles	
				fprintf(htmlfile,"<HR>");				
				for (int i=0;i < curr_unit.nr_mapped_vecs;i++)
				{
  				fprintf(htmlfile,"<a href=%s%s%s>%s%s</a><BR>",keylinkpath,curr_unit.filelist[i].name,keyfileending,curr_unit.filelist[i].name,keyfileending);
				}
			}
		}
	}
	else
	{ // its a knot, so write the link to the mapped som
		memset(mapped_som, 0 , sizeof(mapped_som));
   	strcpy(help_mapped_som, strrchr(curr_unit.url_mapped_soms.name, '/')+1); //extract the filename from path
   	strncpy(mapped_som,help_mapped_som, strlen(help_mapped_som) -4);
   	strcat(mapped_som, "unit.labelunit.html"); // string operations in c is boring shit
		fprintf(htmlfile,"<a href=%s>down</a><BR>", mapped_som);
	}
	
	fprintf(htmlfile,"</td>\n");
	if (curr_unit.pos_x == (curr_map.xdim-1))
	{ //new line in the table
		fprintf(htmlfile,"</tr>\n");
	}
	if ((curr_unit.pos_x==(curr_map.xdim-1)) && (curr_unit.pos_y==(curr_map.ydim-1)))
	{ // write footer
      if (strlen(curr_map.somabove) != 0)
			{ // CREATE THE UPLINK
  			memset(somabove, 0, sizeof(somabove));
  			strcpy(somabove, strrchr(curr_map.somabove, '/')+1);
  			somabove[strlen(somabove)-1] = '\0';
  			fprintf(htmlfile,"<a href=%s.labelunit.html>One Level Up</a><BR><BR>",somabove);		
  		}
		fprintf(htmlfile,"</table></BODY></HTML>\n");
	}
	return 1;
	
}

int calculate_color(double max_value, double min_value, double score)
{
	int return_value;
 	if ( min_value == max_value)
	{
			return 100;
	}	
	else
	{
		return_value =(int) ((((score-min_value)/(max_value-min_value))* MINGRAYVALUE * -1) + MINGRAYVALUE);
 		return return_value;
	}
}

